#include "savefilebase.h"
#include <QFile>
#include <QImage>
#include "../saneobject.h"
#include <QPixmap>
#include <QPainter>
#include <QDebug>
#include <QFile>

constexpr inline int U(const char *str)
{
    return str[0] + (str[1] ? U(str + 1) : 0);
}
SaveFileBase::SaveFileBase(QObject *parent) : QObject(parent)
{

}

void SaveFileBase::saveFileOP(QString filename,QImage *imgOP){
    m_fileName = filename;
    bool res = imageSave(imgOP);
    if (!res) {
        saveToPdf(imgOP);
    }
}
bool SaveFileBase::imageSave(QImage *imgOP)
{
    qDebug() << "save image: " << m_fileName;
    if (g_sane_object->ocrFlag == 0) {
        if (m_fileName.endsWith(".pdf"))
            return false;
        /*****************mark***************************/
        QImage tmp = imgOP->copy();
        /*****************mark***************************/
        if (m_fileName.endsWith(".png") || m_fileName.endsWith(".jpg") || m_fileName.endsWith(".bmp"))
            tmp.save(m_fileName);
        if (m_fileName.endsWith(".tiff")){
            saveAsTiff(tmp, m_fileName);
        }
    } else {
        if (!m_fileName.endsWith(".txt"))
            m_fileName += ".txt";
        QFile file(m_fileName);
        file.open(QIODevice::ReadWrite | QIODevice::Text);
        QByteArray str = g_sane_object->ocrOutputText.toUtf8();
        file.write(str);
        file.close();
    }
    g_user_signal->exitWindowWithSaveFlag = false;
    return true;
}
void SaveFileBase::saveToPdf(QImage *imgOP)
{
    QImage tmp = imgOP->copy();

    QFile pdfFile(m_fileName);
    pdfFile.open(QIODevice::WriteOnly);
    QPdfWriter *pdfWriter = new QPdfWriter(&pdfFile);
    setPdfSize(pdfWriter, g_sane_object->userInfo.size);

    int resolution = g_sane_object->resolutionValue;
    pdfWriter->setResolution(resolution);

    int pageMargin = 0;
    pdfWriter->setPageMargins(QMarginsF(pageMargin, pageMargin, pageMargin, pageMargin));

    QPainter *pdfPainter = new QPainter(pdfWriter);

    int yCurrentP = 0;
    int xCurrentP = 0;

    QPixmap pixmap = QPixmap::fromImage(tmp);
    pdfPainter->drawPixmap(xCurrentP, yCurrentP, pixmap.width(), pixmap.height(), pixmap);

    delete pdfPainter;
    delete pdfWriter;
    pdfFile.close();
    g_user_signal->exitWindowWithSaveFlag = false;
}

void SaveFileBase::setPdfSize(QPdfWriter *pdfWriter, QString size)
{
    switch (toUnicode(size)) {
    case U("A0"):
        pdfWriter->setPageSize(QPageSize(QPageSize::A0));
        break;
    case U("A1"):
        pdfWriter->setPageSize(QPageSize(QPageSize::A1));
        break;
    case U("A2"):
        pdfWriter->setPageSize(QPageSize(QPageSize::A2));
        break;
    case U("A3"):
        pdfWriter->setPageSize(QPageSize(QPageSize::A3));
        break;
    case U("A4"):
        pdfWriter->setPageSize(QPageSize(QPageSize::A4));
        break;
    case U("A5"):
        pdfWriter->setPageSize(QPageSize(QPageSize::A5));
        break;
    case U("A6"):
        pdfWriter->setPageSize(QPageSize(QPageSize::A6));
        break;
    case U("A7"):
        pdfWriter->setPageSize(QPageSize(QPageSize::A7));
        break;
    case U("A8"):
        pdfWriter->setPageSize(QPageSize(QPageSize::A8));
        break;
    case U("A9"):
        pdfWriter->setPageSize(QPageSize(QPageSize::A9));
        break;
    default:
        pdfWriter->setPageSize(QPageSize(QPageSize::A4));
        break;
    }

}

void SaveFileBase::saveAsTiff(QImage image, QString file_name)
{
    m_tiffImageList.append(image);
    m_fileImages.append(file_name);
    image.save(file_name);

    QString save_name = g_sane_object->saveFullScanFileName + ".tif";
    if(!QFile::exists(save_name)){
        m_tiffImageList[0].save(save_name);
    }

    FREE_IMAGE_FORMAT fif = FIF_TIFF;
    FIBITMAP *page;
    int s = g_sane_object->scanPageNumber;
    if(m_tiffImageList.length() == g_sane_object->scanPageNumber){

        FIMULTIBITMAP *src = FreeImage_OpenMultiBitmap(fif, save_name.toLocal8Bit().data(), false, false, true, 0);
        for(int i = 1; i < m_tiffImageList.length(); i++){
            QString newFileName = m_fileImages[i];
            FIMULTIBITMAP *newfile = FreeImage_OpenMultiBitmap(fif, newFileName.toLocal8Bit().data(), false, true);
            page = FreeImage_LockPage(newfile, 0);
            m_filist.append(FreeImage_Clone(page));
            FreeImage_UnlockPage(newfile, page, 0);
        }
        FreeImage_CloseMultiBitmap(src);
    }

    if(m_filist.length() == (g_sane_object->scanPageNumber - 1)){
        FIMULTIBITMAP *out = FreeImage_OpenMultiBitmap(fif, save_name.toLocal8Bit().data(), false, false);
        for(int i = 0; i < m_filist.length(); i++){
            FreeImage_AppendPage(out, m_filist.at(i));
        }
        bool result = FreeImage_CloseMultiBitmap(out);

        if(result){
            for(int j = 0; j < m_fileImages.length(); j++){
                QString filename = m_fileImages.at(j);
                QFile *a = new QFile(filename);
                a->remove();
                a->deleteLater();
            }
            m_tiffImageList.clear();
            m_fileImages.clear();
        }
    }
}

int SaveFileBase::toUnicode(QString str)
{
    char  *ch;
    QByteArray ba = str.toLatin1();
    ch = ba.data();
    return ch[0] + (ch[1] ? toUnicode(ch + 1) : 0);

}
