#ifndef SHOWIMAGEWIDGET_H
#define SHOWIMAGEWIDGET_H

#include <QWidget>
#include <QWidgetList>
#include <QStackedWidget>
#include <QLabel>
#include <QPainter>
#include <QImage>
#include <QMenu>
#include <QAction>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QImage>
#include <QFile>
#include <QPdfWriter>
#include <QResizeEvent>
#include <QKeyEvent>
#include <QStack>
#include <QMatrix>
#include <QThread>
#include "imageBaseOP/loadimage.h"
#include <QMovie>
#include <QStackedLayout>

#include <opencv4/opencv2/core.hpp>
#include <opencv4/opencv2/imgcodecs.hpp>
#include <opencv4/opencv2/imgproc.hpp>
using namespace cv;

//#include <kylin_image_codec/kylinimagecodec.h>

#include "globalsignal.h"
#include "saneobject.h"
#include "toolbarwidget.h"
#include "watermarkdialog.h"
#include "crop.h"
#include "beauty.h"
#include "rectify.h"
#include "include/common.h"
#include "utils/copyoperation.h"
#include "imageOp/imageoperationbase.h"
#include "imageBaseOP/savefilebase.h"
#include <kborderlessbutton.h>

#define ShowImageWidgetMinimumSize QSize(387, 536)
#define CancelOkWidgetFixedSize QSize(92, 36)


class ShowImageWidget : public QWidget
{
    Q_OBJECT
public:
    explicit ShowImageWidget(QWidget *parent = nullptr);
    friend class LoadImage;

    void rollBackOperation();

    void setupGui();
    void initConnect();

    int toUnicode(QString str);
    void setPdfSize(QPdfWriter *pdfWriter, QString size);
    void saveToPdf(QImage img, QString pathname);

    QImage *imageSave(QString filename);

    QString setPixmapScaled(QImage img, QLabel *lab, double scale = 1.0);

    void setPixmapScaledByProportion(double scaledNumber);

    enum ZoomStatus {
        ZoomOut,
        ZoomIn
    };

    void getCropArea(int &x1, int &y1, int &x2, int &y2);

    double getCurrentPercentage();
    double getRealPercentage(ZoomStatus zoomType);
    void updatePercentageByZoom(double scale);

    const QString scannerImagePath;
    const int defaultImageLableWidth = 387;
    const int defaultImageLableHeight = 536;

    QSize defaultScanImageSize;
    QImage *m_stackImage;
    QImage *m_editImage;
    QImage *m_normalImage;
    QList<QImage*> m_normalImageList;
    QPixmap m_normalPixmap;
    QStack<QImage> m_imageStack;


protected:
    void resizeEvent(QResizeEvent *event) Q_DECL_OVERRIDE;
    void keyPressEvent(QKeyEvent *event) Q_DECL_OVERRIDE;
    bool eventFilter(QObject *obj, QEvent *event) Q_DECL_OVERRIDE;
public slots:
    void beautyFinished();
    void rotateFinished();
    void cancelImageOperationSlot();
    void loadImageFinishedSlot(QString, double, QImage);

    void showNormalImageAfterScan(QStringList, QStringList);
    QPixmap resizePix(const QPixmap &pixmap , const QSize &size);

    void showImageAfterClickedThumbnail(QString loadPath);

    void saveImage(QString filename);

    void cropSlot();
    void rotateSlot();
    void mirrorSlot();
    void watermarkSlot();
    void zoomoutNormalImage();
    void zoominNormalImage();
    void zoomCurrentNormalImage();

    void beautyStartSlot();
    void beautyStopSlot();
    void rectifyStartSlot();
    void rectifyStopSlot(bool isTrue);
    void ocrStartSlot();
    void ocrStopSlot();

    void saveCurrentPicture();
    void loadAfterBROPicture();

    void themeChangedSlot();
    void cropCancelSlot();
    void cropCompleteSlot();

private:
    QString m_loadPathDelayUpdate;
    QString m_savePathDelayUpdate;
    bool showImageAfterClickedThumbnailFlag = false;
    RunningDialog *m_beautyRunningDialog = nullptr;
    void showBeautyRunningDialog(QString text);
    void imageNUll(bool res);

    bool rotationFlag = false;
    double proportion = 1.0;
    double proportionForPercentage = 1.0;

    Mat matImage;

    QLabel *m_showImageLabel = nullptr;
    QLabel *m_waitingImageLabel = nullptr;
    QStackedLayout *m_stackedLayout = nullptr;
    QWidget *m_labelWidget = nullptr;

    CropLabel *m_cropLabel;
    QStackedWidget *m_showImageAndCropWidget;
    QHBoxLayout *m_showImageHLayout;

    QWidget *m_cancelOkWidget;
    QHBoxLayout *m_cancelOkLayout;
    QPushButton *m_cancelButton;
    QPushButton *m_okButton;

    ToolBarWidget *m_toolbarWidget;

    CopyOperation *toolObject = nullptr;

    QVBoxLayout *m_mainVLayout;
    ImageOperationBase *imageOp = nullptr;
    QThread *imageOpThread = nullptr;

    LoadImage *loadImg = nullptr;
    QThread loadImgOP;
    QMovie *m_loadingMovie = nullptr;
    SaveFileBase m_saveFileBase;

    void setCancelWidgetPositon();
    void drawShadow();
signals:
    void beautySignal();
    void rectifySignal();
    void OCRSignal();
    void rotateSignal();
    void mirrorSignal();

    void loadImgSignal(QString loadPath, QString savePath, QImage *img, QLabel *container,double scale, bool rotationFlag,QSize labSize);

};

#endif // SHOWIMAGEWIDGET_H
