#include "toolbarwidget.h"
#include <buried_point.hpp>
#include <gsettings.hpp>
#include "include/theme.h"
#include <QGraphicsDropShadowEffect>
#include <QMessageBox>

ToolBarWidget::ToolBarWidget(QWidget *parent) : QWidget(parent),
    m_beautyButton(new kdk::KBorderlessButton),
    m_rectifyButton(new kdk::KBorderlessButton),
    m_ocrButton(new kdk::KBorderlessButton),
    m_leftFrame(new QFrame),
    m_cropButton(new kdk::KBorderlessButton),
    m_rotateButton(new kdk::KBorderlessButton),
    m_mirrorButton(new kdk::KBorderlessButton),
    m_watermarkButton(new kdk::KBorderlessButton),
    m_rightFrame(new QFrame),
    m_zoomOutButton(new kdk::KBorderlessButton),
    m_percentageLabel(new QLabel),
    m_zoomInButton(new kdk::KBorderlessButton),
    m_mainHLayout(new QHBoxLayout(this))
{
    setupGui();
    initConnect();
}

void ToolBarWidget::setupGui()
{
    this->setAttribute(Qt::WA_TranslucentBackground); // set background translucent，combine with overriding paintEvent to let setStyleSheet work
    this->setWindowFlags(Qt::FramelessWindowHint);   // set frameless window
    setAutoFillBackground(true);
    setBackgroundRole(QPalette::Base);

    deviceMode currentMode = m_mode.defaultModeCapture();
    if(currentMode != PCMode){
        this->setFixedSize(ToolBarWidgetTableModeSize);

        m_beautyButton->setIcon(QIcon::fromTheme(":/toolbar/beauty.svg"));
        m_beautyButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_beautyButton->setIconSize(ToolBarWidgetTableModeButtonSize);
        m_beautyButton->setToolTip(tr("Beauty"));
        m_beautyButton->setFocusPolicy(Qt::NoFocus);

        m_rectifyButton->setIcon(QIcon::fromTheme(":/toolbar/rectify.svg"));
        m_rectifyButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_rectifyButton->setIconSize(ToolBarWidgetTableModeButtonSize);
        m_rectifyButton->setToolTip(tr("Rectify"));
        m_rectifyButton->setFocusPolicy(Qt::NoFocus);

        m_ocrButton->setIcon(QIcon::fromTheme(":/toolbar/ocr.svg"));
        m_ocrButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_ocrButton->setIconSize(ToolBarWidgetTableModeButtonSize);
        m_ocrButton->setToolTip(tr("OCR"));
        m_ocrButton->setFocusPolicy(Qt::NoFocus);

        m_leftFrame->setFixedSize(ToolBarWidgetFrameSize);

        m_cropButton->setIcon(QIcon::fromTheme(":/toolbar/crop.svg"));
        m_cropButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_cropButton->setIconSize(ToolBarWidgetTableModeButtonSize);
        m_cropButton->setToolTip(tr("Crop"));
        m_cropButton->setFocusPolicy(Qt::NoFocus);

        m_rotateButton->setIcon(QIcon::fromTheme(":/toolbar/rotate.svg"));
        m_rotateButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_rotateButton->setIconSize(ToolBarWidgetTableModeButtonSize);
        m_rotateButton->setToolTip(tr("Rotate"));
        m_rotateButton->setFocusPolicy(Qt::NoFocus);

        m_mirrorButton->setIcon(QIcon::fromTheme(":/toolbar/mirror.svg"));
        m_mirrorButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_mirrorButton->setIconSize(ToolBarWidgetTableModeButtonSize);
        m_mirrorButton->setToolTip(tr("Mirror"));
        m_mirrorButton->setFocusPolicy(Qt::NoFocus);

        m_watermarkButton->setIcon(QIcon::fromTheme(":/toolbar/watermark.svg"));
        m_watermarkButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_watermarkButton->setIconSize(ToolBarWidgetTableModeButtonSize);
        m_watermarkButton->setToolTip(tr("Watermark"));
        m_watermarkButton->setFocusPolicy(Qt::NoFocus);

        m_rightFrame->setFixedSize(ToolBarWidgetFrameSize);

        QFont m_percentageFontSize;
        m_percentageFontSize.setPixelSize(14);
        m_percentageLabel->setText("100%");
        m_percentageLabel->setFont(m_percentageFontSize);

        m_zoomOutButton->setIcon(QIcon::fromTheme(":/toolbar/zoomout.svg"));
        m_zoomOutButton->setFixedSize(ToolBarWidgetTableModeZoomButtonSize);
        m_zoomOutButton->setIconSize(ToolBarWidgetTableModeZoomButtonSize);
        m_zoomOutButton->setToolTip(tr("ZoomOut"));
        m_zoomOutButton->setFocusPolicy(Qt::NoFocus);

        m_zoomInButton->setIcon(QIcon::fromTheme(":/toolbar/zoomin.svg"));
        m_zoomInButton->setFixedSize(ToolBarWidgetTableModeZoomButtonSize);
        m_zoomInButton->setIconSize(ToolBarWidgetTableModeZoomButtonSize);
        m_zoomInButton->setToolTip(tr("ZoomIn"));
        m_zoomInButton->setFocusPolicy(Qt::NoFocus);


        m_mainHLayout->setSpacing(0);
        m_mainHLayout->addWidget(m_beautyButton);
        m_mainHLayout->addSpacing(4);
        m_mainHLayout->addWidget(m_rectifyButton);
        m_mainHLayout->addSpacing(4);
        m_mainHLayout->addWidget(m_ocrButton);
        m_mainHLayout->addSpacing(8);
        m_mainHLayout->addWidget(m_leftFrame);
        m_mainHLayout->addSpacing(4);
        m_mainHLayout->addWidget(m_cropButton);
        m_mainHLayout->addSpacing(4);
        m_mainHLayout->addWidget(m_rotateButton);
        m_mainHLayout->addSpacing(4);
        m_mainHLayout->addWidget(m_mirrorButton);
        m_mainHLayout->addSpacing(3);
        m_mainHLayout->addWidget(m_watermarkButton);
        m_mainHLayout->addSpacing(8);
        m_mainHLayout->addWidget(m_rightFrame);
        m_mainHLayout->addSpacing(16);
        m_mainHLayout->addWidget(m_zoomOutButton, 0, Qt::AlignCenter);
        m_mainHLayout->addSpacing(6);
        m_mainHLayout->addWidget(m_percentageLabel, 0, Qt::AlignCenter);
        m_mainHLayout->addSpacing(4);
        m_mainHLayout->addWidget(m_zoomInButton, 0, Qt::AlignCenter);
        m_mainHLayout->setContentsMargins(8, 0, 16, 0);

        this->setLayout(m_mainHLayout);
    }else{
        this->setFixedSize(ToolBarWidgetFixedSize);

        m_beautyButton->setIcon(QIcon::fromTheme(":/toolbar/beauty.svg"));
        m_beautyButton->setFixedSize(ToolBarWidgetButtonSize);
        m_beautyButton->setIconSize(ToolBarWidgetButtonSize);
        m_beautyButton->setToolTip(tr("Beauty"));
        m_beautyButton->setFocusPolicy(Qt::NoFocus);

        m_rectifyButton->setIcon(QIcon::fromTheme(":/toolbar/rectify.svg"));
        m_rectifyButton->setFixedSize(ToolBarWidgetButtonSize);
        m_rectifyButton->setIconSize(ToolBarWidgetButtonSize);
        m_rectifyButton->setToolTip(tr("Rectify"));
        m_rectifyButton->setFocusPolicy(Qt::NoFocus);

        m_ocrButton->setIcon(QIcon::fromTheme(":/toolbar/ocr.svg"));
        m_ocrButton->setFixedSize(ToolBarWidgetButtonSize);
        m_ocrButton->setIconSize(ToolBarWidgetButtonSize);
        m_ocrButton->setToolTip(tr("OCR"));
        m_ocrButton->setFocusPolicy(Qt::NoFocus);

        m_leftFrame->setFixedSize(ToolBarWidgetFrameSize);

        m_cropButton->setIcon(QIcon::fromTheme(":/toolbar/crop.svg"));
        m_cropButton->setFixedSize(ToolBarWidgetButtonSize);
        m_cropButton->setIconSize(ToolBarWidgetButtonSize);
        m_cropButton->setToolTip(tr("Crop"));
        m_cropButton->setFocusPolicy(Qt::NoFocus);

        m_rotateButton->setIcon(QIcon::fromTheme(":/toolbar/rotate.svg"));
        m_rotateButton->setFixedSize(ToolBarWidgetButtonSize);
        m_rotateButton->setIconSize(ToolBarWidgetButtonSize);
        m_rotateButton->setToolTip(tr("Rotate"));
        m_rotateButton->setFocusPolicy(Qt::NoFocus);

        m_mirrorButton->setIcon(QIcon::fromTheme(":/toolbar/mirror.svg"));
        m_mirrorButton->setFixedSize(ToolBarWidgetButtonSize);
        m_mirrorButton->setIconSize(ToolBarWidgetButtonSize);
        m_mirrorButton->setToolTip(tr("Mirror"));
        m_mirrorButton->setFocusPolicy(Qt::NoFocus);

        m_watermarkButton->setIcon(QIcon::fromTheme(":/toolbar/watermark.svg"));
        m_watermarkButton->setFixedSize(ToolBarWidgetButtonSize);
        m_watermarkButton->setIconSize(ToolBarWidgetButtonSize);
        m_watermarkButton->setToolTip(tr("Watermark"));
        m_watermarkButton->setFocusPolicy(Qt::NoFocus);

        m_rightFrame->setFixedSize(ToolBarWidgetFrameSize);

        QFont m_percentageFontSize;
        m_percentageFontSize.setPixelSize(12);
        m_percentageLabel->setText("100%");
        m_percentageLabel->setFont(m_percentageFontSize);

        m_zoomOutButton->setIcon(QIcon::fromTheme(":/toolbar/zoomout.svg"));
        m_zoomOutButton->setFixedSize(ToolBarWidgetZoomButtonSize);
        m_zoomOutButton->setIconSize(ToolBarWidgetZoomButtonSize);
        m_zoomOutButton->setToolTip(tr("ZoomOut"));
        m_zoomOutButton->setFocusPolicy(Qt::NoFocus);

        m_zoomInButton->setIcon(QIcon::fromTheme(":/toolbar/zoomin.svg"));
        m_zoomInButton->setFixedSize(ToolBarWidgetZoomButtonSize);
        m_zoomInButton->setIconSize(ToolBarWidgetZoomButtonSize);
        m_zoomInButton->setToolTip(tr("ZoomIn"));
        m_zoomInButton->setFocusPolicy(Qt::NoFocus);


        m_mainHLayout->setSpacing(0);
        m_mainHLayout->addWidget(m_beautyButton);
        m_mainHLayout->addSpacing(4);
        m_mainHLayout->addWidget(m_rectifyButton);
        m_mainHLayout->addSpacing(4);
        m_mainHLayout->addWidget(m_ocrButton);
        m_mainHLayout->addSpacing(8);
        m_mainHLayout->addWidget(m_leftFrame);
        m_mainHLayout->addSpacing(4);
        m_mainHLayout->addWidget(m_cropButton);
        m_mainHLayout->addSpacing(4);
        m_mainHLayout->addWidget(m_rotateButton);
        m_mainHLayout->addSpacing(4);
        m_mainHLayout->addWidget(m_mirrorButton);
        m_mainHLayout->addSpacing(3);
        m_mainHLayout->addWidget(m_watermarkButton);
        m_mainHLayout->addSpacing(8);
        m_mainHLayout->addWidget(m_rightFrame);
        m_mainHLayout->addSpacing(16);
        m_mainHLayout->addWidget(m_zoomOutButton, 0, Qt::AlignCenter);
        m_mainHLayout->addSpacing(6);
        m_mainHLayout->addWidget(m_percentageLabel, 0, Qt::AlignCenter);
        m_mainHLayout->addSpacing(4);
        m_mainHLayout->addWidget(m_zoomInButton, 0, Qt::AlignCenter);
        m_mainHLayout->setContentsMargins(8, 0, 16, 0);

        this->setLayout(m_mainHLayout);
    }

    drawFrame();
    themeChangedSlot();

    KyInfo() << "toolbarwidget size: " << this->size();

//    QGraphicsDropShadowEffect *effect = new QGraphicsDropShadowEffect(this);
//    effect->setOffset(0, 0);          // set direction of shadow effect, especially (0,0) is on behave of spread around
//    effect->setColor(QColor(220, 220, 220)); // set shadow color
//    effect->setBlurRadius(20);        // set blur of shadow, which means the large the radius, the more blurred
//    this->setGraphicsEffect(effect);

}

void ToolBarWidget::initConnect()
{
    connect(kdk::kabase::Gsettings::getPoint(), &kdk::kabase::Gsettings::systemThemeChange, this, &ToolBarWidget::themeChangedSlot);

    connect(g_user_signal, &GlobalUserSignal::toolbarPercentageChangedSignel, this, &ToolBarWidget::percentageChangedSlot);

    connect(m_beautyButton, &QPushButton::clicked, this, &ToolBarWidget::beautyButtonClickedSlot);
    connect(m_rectifyButton, &QPushButton::clicked, this, &ToolBarWidget::rectifyButtonClickedSlot);
    connect(m_ocrButton, &QPushButton::clicked, this, &ToolBarWidget::ocrButtonClickedSlot);
    connect(m_cropButton, &QPushButton::clicked, this, &ToolBarWidget::cropButtonClickedSlot);
    connect(m_rotateButton, &QPushButton::clicked, this, &ToolBarWidget::rotateButtonClickedSlot);
    connect(m_mirrorButton, &QPushButton::clicked, this, &ToolBarWidget::mirrorButtonClickedSlot);
    connect(m_watermarkButton, &QPushButton::clicked, this, &ToolBarWidget::watermarkButtonClickedSlot);
    connect(m_zoomOutButton, &QPushButton::clicked, this, &ToolBarWidget::zoomoutButtonClickedSlot);
    connect(m_zoomInButton, &QPushButton::clicked, this, &ToolBarWidget::zoominButtonClickedSlot);
    connect(GlobalUserSignal::getInstance(), &GlobalUserSignal::rotationChangedSig, this, &ToolBarWidget::rotateChangedSlot);
}
void ToolBarWidget::themeChangedSlot(){
    QString systemTheme = kdk::kabase::Gsettings::getSystemTheme().toString();

    if (systemTheme == STYLE_NAME_KEY_DARK || systemTheme == STYLE_NAME_KEY_BLACK) {
        KyInfo() << "dark theme";
        this->setStyleSheet("background: rgba(38, 38, 38, 1); box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.15); border-radius: 6px; filter: blur(18px);");
    } else {
        KyInfo() << "white theme";
        this->setStyleSheet("background: rgba(249, 249, 249, 0.8); box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.15); border-radius: 6px; filter: blur(18px);");
    }
}

void ToolBarWidget::rotateChangedSlot(bool isPCMode)
{
    if(!isPCMode){
        this->setFixedSize(ToolBarWidgetTableModeSize);

        m_beautyButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_beautyButton->setIconSize(ToolBarWidgetTableModeButtonSize);

        m_rectifyButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_rectifyButton->setIconSize(ToolBarWidgetTableModeButtonSize);

        m_ocrButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_ocrButton->setIconSize(ToolBarWidgetTableModeButtonSize);

        m_leftFrame->setFixedSize(ToolBarWidgetFrameSize);

        m_cropButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_cropButton->setIconSize(ToolBarWidgetTableModeButtonSize);

        m_rotateButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_rotateButton->setIconSize(ToolBarWidgetTableModeButtonSize);

        m_mirrorButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_mirrorButton->setIconSize(ToolBarWidgetTableModeButtonSize);

        m_watermarkButton->setFixedSize(ToolBarWidgetTableModeButtonSize);
        m_watermarkButton->setIconSize(ToolBarWidgetTableModeButtonSize);

        m_rightFrame->setFixedSize(ToolBarWidgetFrameSize);

        QFont m_percentageFontSize;
        m_percentageFontSize.setPixelSize(12);
        m_percentageLabel->setFont(m_percentageFontSize);

        m_zoomOutButton->setFixedSize(ToolBarWidgetTableModeZoomButtonSize);
        m_zoomOutButton->setIconSize(ToolBarWidgetTableModeZoomButtonSize);

        m_zoomInButton->setFixedSize(ToolBarWidgetTableModeZoomButtonSize);
        m_zoomInButton->setIconSize(ToolBarWidgetTableModeZoomButtonSize);
    }else{
        this->setFixedSize(ToolBarWidgetFixedSize);

        m_beautyButton->setFixedSize(ToolBarWidgetButtonSize);
        m_beautyButton->setIconSize(ToolBarWidgetButtonSize);

        m_rectifyButton->setFixedSize(ToolBarWidgetButtonSize);
        m_rectifyButton->setIconSize(ToolBarWidgetButtonSize);

        m_ocrButton->setFixedSize(ToolBarWidgetButtonSize);
        m_ocrButton->setIconSize(ToolBarWidgetButtonSize);

        m_leftFrame->setFixedSize(ToolBarWidgetFrameSize);

        m_cropButton->setFixedSize(ToolBarWidgetButtonSize);
        m_cropButton->setIconSize(ToolBarWidgetButtonSize);

        m_rotateButton->setFixedSize(ToolBarWidgetButtonSize);
        m_rotateButton->setIconSize(ToolBarWidgetButtonSize);

        m_mirrorButton->setFixedSize(ToolBarWidgetButtonSize);
        m_mirrorButton->setIconSize(ToolBarWidgetButtonSize);

        m_watermarkButton->setFixedSize(ToolBarWidgetButtonSize);
        m_watermarkButton->setIconSize(ToolBarWidgetButtonSize);

        m_rightFrame->setFixedSize(ToolBarWidgetFrameSize);

        QFont m_percentageFontSize;
        m_percentageFontSize.setPixelSize(14);
        m_percentageLabel->setFont(m_percentageFontSize);

        m_zoomOutButton->setFixedSize(ToolBarWidgetZoomButtonSize);
        m_zoomOutButton->setIconSize(ToolBarWidgetZoomButtonSize);

        m_zoomInButton->setFixedSize(ToolBarWidgetZoomButtonSize);
        m_zoomInButton->setIconSize(ToolBarWidgetZoomButtonSize);
    }
}

void ToolBarWidget::drawFrame()
{
    QGraphicsDropShadowEffect *effect = new QGraphicsDropShadowEffect(this);
    effect->setOffset(0, 0); //设置向哪个方向产生阴影效果(dx,dy)，特别地，(0,0)代表向四周发散
    effect->setColor(QColor(220,220,220));       //设置阴影颜色，也可以setColor()
    effect->setBlurRadius(8); //设定阴影的模糊半径，数值越大越模糊
    this->setGraphicsEffect(effect);
}

void ToolBarWidget::ocrOtherIconState(bool flag)
{
    m_beautyButton->setDisabled(flag);
    m_rectifyButton->setDisabled(flag);
    m_cropButton->setDisabled(flag);
    m_rotateButton->setDisabled(flag);
    m_mirrorButton->setDisabled(flag);
    m_watermarkButton->setDisabled(flag);
    m_zoomOutButton->setDisabled(flag);
    m_zoomInButton->setDisabled(flag);
    m_percentageLabel->setDisabled(flag);
}

/**
 * @brief ToolBarWidget::paintEvent
 * @param event
 * Basically, it only support these stylesheet attributes for QWidget : background, background-clip and background-origin
 */
void ToolBarWidget::paintEvent(QPaintEvent *event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter painter(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &painter, this);

    QWidget::paintEvent(event);

}

void ToolBarWidget::keyPressEvent(QKeyEvent *event)
{
    Q_UNUSED(event);

    return;
}

void ToolBarWidget::keyReleaseEvent(QKeyEvent *event)
{
    Q_UNUSED(event);

    return;
}

void ToolBarWidget::themeChangedWhiteSlot()
{


//    themeChangedIconWhiteSettings();

    this->setStyleSheet("background: rgba(249, 249, 249, 0.8); box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.15); border-radius: 6px; filter: blur(18px);");

//    m_leftFrame->setStyleSheet("background: #B3B3B3; box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.15);");
//    m_rightFrame->setStyleSheet("background: #B3B3B3; box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.15);");


//    m_beautyButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/beauty.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/beauty-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/beauty-blue.svg);}");

//    m_rectifyButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/rectify.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/rectify-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/rectify-blue.svg);}");

//    m_ocrButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/ocr.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/ocr-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/ocr-blue.svg);}");

//    m_cropButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/crop.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/crop-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/crop-blue.svg);}");

//    m_rotateButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/rotate.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/rotate-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/rotate-blue.svg);}");

//    m_mirrorButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/mirror.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/mirror-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/mirror-blue.svg);}");

//    m_watermarkButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/watermark.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/watermark-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/watermark-blue.svg);}");


//    m_zoomOutButton->setStyleSheet("background: #FFFFFF; box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.15); border-radius: 10px;");
//    m_percentageLabel->setStyleSheet("width: 32px; height: 18px; color: #262626;");
//    m_zoomInButton->setStyleSheet("background: #FFFFFF; box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.15); border-radius: 10px;");
}

void ToolBarWidget::themeChangedBlackSlot()
{
//    themeChangedIconBlackSettings();

    this->setStyleSheet("background: rgba(38, 38, 38, 1); box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.15); border-radius: 6px; filter: blur(18px);");

//    m_leftFrame->setStyleSheet("background: #B3B3B3; box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.15);");
//    m_rightFrame->setStyleSheet("background: #B3B3B3; box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.15);");

//    m_beautyButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/beauty-white.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/beauty-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/beauty-blue.svg);}");

//    m_rectifyButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/rectify-white.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/rectify-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/rectify-blue.svg);}");

//    m_ocrButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/ocr-white.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/ocr-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/ocr-blue.svg);}");

//    m_cropButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/crop-white.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/crop-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/crop-blue.svg);}");

//    m_rotateButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/rotate-white.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/rotate-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/rotate-blue.svg);}");

//    m_mirrorButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/mirror-white.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/mirror-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/mirror-blue.svg);}");

//    m_watermarkButton->setStyleSheet("QPushButton{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/watermark-white.svg);}"
//                                   "QPushButton::hover{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/watermark-blue.svg);}"
//                                   "QPushButton::pressed{border:0px; border-radius: 6px; background:transparent; background-image:url(:/toolbar/watermark-blue.svg);}");



//    m_zoomOutButton->setStyleSheet("background: #373738; box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.15); border-radius: 10px;");
//    m_percentageLabel->setStyleSheet("width: 32px; height: 18px; color: #D9D9D9;");
//    m_zoomInButton->setStyleSheet("background: #373738; box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.15); border-radius: 10px;");

}

void ToolBarWidget::percentageChangedSlot()
{
    KyInfo() << "percentage = " << g_sane_object->percentage;

    m_percentageLabel->setText(g_sane_object->percentage);
}
void ToolBarWidget::beautyButtonClickedSlot()
{
    KyInfo() << "click beaty button.";

    if(g_sane_object->ocrFlag || g_sane_object->cropFlag){
        qDebug() << "do nothing,exit beauty!";
        return;
    }
    g_user_signal->exitWindowWithSaveFlag = true;
    g_user_signal->toolbarBeautyOperationStart();
    qDebug() << "show beauty dialog!";
    qDebug() << "continue!";

    kdk::kabase::BuriedPoint buriedPointTest;

    if (buriedPointTest.functionBuriedPoint(kdk::kabase::AppName::KylinScanner , kdk::kabase::BuriedPoint::PT::KylinScannerOneClickBeautification)) {
        qCritical() << "Error : buried point fail !";
    };
}

void ToolBarWidget::rectifyButtonClickedSlot()
{
    KyInfo() << "click rectify button.";
    if(g_sane_object->ocrFlag || g_sane_object->cropFlag){
        qDebug() << "do nothing,exit rectify!";
        return;
    }
    g_user_signal->exitWindowWithSaveFlag = true;
    g_user_signal->toolbarRectifyOperationStart();
    kdk::kabase::BuriedPoint buriedPointTest;

    if (buriedPointTest.functionBuriedPoint(kdk::kabase::AppName::KylinScanner , kdk::kabase::BuriedPoint::PT::KylinScannerRectification)) {
        qCritical() << "Error : buried point fail !";
    };
}

void ToolBarWidget::ocrButtonClickedSlot()
{
    KyInfo() << "click ocr button.";

    if(g_sane_object->cropFlag){
        qDebug() << "do nothing,exit ocr!";
        return;
    }
    emit g_user_signal->switchPageSig();
    if(g_sane_object->ocrFlag){
        ocrOtherIconState(true);
        qDebug() << "exit OCR page...";
        emit g_user_signal->exitOCR();
        g_sane_object->ocrFlag = 0;
    }else{
        ocrOtherIconState(false);
        g_user_signal->toolbarOcrOperationStart();
        kdk::kabase::BuriedPoint buriedPointTest;
        if (buriedPointTest.functionBuriedPoint(kdk::kabase::AppName::KylinScanner , kdk::kabase::BuriedPoint::PT::KylinScannerTextRecognition)) {
            qCritical() << "Error : buried point fail !";
        };
        g_sane_object->ocrFlag = 1;
    }
}

void ToolBarWidget::cropButtonClickedSlot()
{
    KyInfo() << "click crop button.";

    if(g_sane_object->ocrFlag){
        qDebug() << "do nothing,exit crop!";
        return;
    }

    g_user_signal->exitWindowWithSaveFlag = true;
    g_user_signal->toolbarCropOperation();
    kdk::kabase::BuriedPoint buriedPointTest;

    if (buriedPointTest.functionBuriedPoint(kdk::kabase::AppName::KylinScanner , kdk::kabase::BuriedPoint::PT::KylinScannerCutting)) {
        qCritical() << "Error : buried point fail !";
    };
}

void ToolBarWidget::rotateButtonClickedSlot()
{
    KyInfo() << "click rotate button.";

    if(g_sane_object->ocrFlag || g_sane_object->cropFlag){
        qDebug() << "do nothing,exit rotate!";
        return;
    }

    g_user_signal->exitWindowWithSaveFlag = true;
    g_user_signal->toolbarRotateOperation();
    kdk::kabase::BuriedPoint buriedPointTest;

    if (buriedPointTest.functionBuriedPoint(kdk::kabase::AppName::KylinScanner , kdk::kabase::BuriedPoint::PT::KylinScannerRotate)) {
        qCritical() << "Error : buried point fail !";
    };
}

void ToolBarWidget::mirrorButtonClickedSlot()
{
    KyInfo() << "click mirror button.";

    if(g_sane_object->ocrFlag || g_sane_object->cropFlag){
        qDebug() << "do nothing,exit mirror!";
        return;
    }

    g_user_signal->exitWindowWithSaveFlag = true;
    g_user_signal->toolbarMirrorOperation();
    kdk::kabase::BuriedPoint buriedPointTest;

    if (buriedPointTest.functionBuriedPoint(kdk::kabase::AppName::KylinScanner , kdk::kabase::BuriedPoint::PT::KylinScannerImage)) {
        qCritical() << "Error : buried point fail !";
    };
}

void ToolBarWidget::watermarkButtonClickedSlot()
{
    KyInfo() << "click watermark button.";

    if(g_sane_object->ocrFlag || g_sane_object->cropFlag){
        qDebug() << "do nothing,exit watermark!";
        return;
    }

    g_user_signal->exitWindowWithSaveFlag = true;
    g_user_signal->toolbarWatermarkOperation();
    kdk::kabase::BuriedPoint buriedPointTest;

    if (buriedPointTest.functionBuriedPoint(kdk::kabase::AppName::KylinScanner , kdk::kabase::BuriedPoint::PT::KylinScannerAddWatermark)) {
        qCritical() << "Error : buried point fail !";
    };
}

void ToolBarWidget::zoomoutButtonClickedSlot()
{
    KyInfo() << "click zoomout button.";

    if(g_sane_object->ocrFlag || g_sane_object->cropFlag){
        qDebug() << "do nothing,exit zooout!";
        return;
    }

    g_user_signal->toolbarZoomoutOperation();
}

void ToolBarWidget::zoominButtonClickedSlot()
{
    KyInfo() << "click zoomin button.";

    if(g_sane_object->ocrFlag || g_sane_object->cropFlag){
        qDebug() << "do nothing,exit zoomin!";
        return;
    }

    g_user_signal->toolbarZoominOperation();
}
